#include <stdlib.h>
#include "global.h"
#include "btstack/debug.h"
#include "btstack/btstack.h"
#include <btstack/hci_cmds.h>
#include <btstack/run_loop.h>

#include "btstack/hci.h"
#include "btstack/l2cap.h"
#include "btstack/btstack_memory.h"
#include "btstack/remote_device_db.h"
//#include "btstack/config.h"

unsigned	timer_counter=0;
unsigned	led1_on_count=0;
uint8_t		led1_on_state=0;
uint8_t		startup_state=0;
WORD		advalue[2];

hci_transport_t * hci_transport_picusb_instance();

#if defined(__PIC24FJ64GB004__)
	_CONFIG1(WDTPS_PS2048 & FWPSA_PR128 & WINDIS_OFF & FWDTENDEF & ICSDEF & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
	_CONFIG2(POSCMOD_NONE & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_FRCPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
	_CONFIG3(WPFP_WPFP0 & SOSCSEL_IO & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
	_CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)
#else

    #error Cannot define configuration bits.

#endif

BYTE        deviceAddress;  // Address of the device on the USB
bd_addr_t addr_global;

BOOL InitializeSystem ( void )
{
   #if defined(__PIC24FJ64GB004__)
	    {
	        unsigned int pll_startup_counter = 600;
	        CLKDIVbits.PLLEN = 1;
	        while(pll_startup_counter--);
	    }

		AD1PCFG = 0xffff;
	    CLKDIV = 0;    // Set PLL prescaler (1:1)
    #endif

	on_setup();
	led1_setup();
	led2_setup();
	u1rx_setup();
	u1tx_setup();
	u1cts_setup();
	u1rts_setup();
	//assoc_setup();
	//pairen_setup();

    // Init UART
    UART1Init(BAUDRATE1);
#if	STO_SERIAL
	sto_setup();
    UART2Init();
#endif

#if	__DEBUG
#else
	OpenADC10(
		ADC_MODULE_ON|ADC_IDLE_CONTINUE|ADC_FORMAT_INTG|ADC_CLK_AUTO|ADC_AUTO_SAMPLING_ON|ADC_SAMP_ON,
		ADC_VREF_AVDD_AVSS|ADC_SCAN_ON|/* ADC_INTR_6_CONV| */ ADC_INTR_2_CONV|ADC_ALT_BUF_OFF|ADC_ALT_INPUT_OFF,
		ADC_CONV_CLK_SYSTEM|ADC_SAMPLE_TIME_31|ADC_CONV_CLK_128Tcy,
		/*ENABLE_AN0_ANA|ENABLE_AN1_ANA|*/ENABLE_AN2_ANA|ENABLE_AN3_ANA/*|ENABLE_AN4_ANA|ENABLE_AN5_ANA|ENABLE_AN6_ANA|ENABLE_AN7_ANA*/,
		/*ADC_SCAN_AN0|ADC_SCAN_AN1|*/ADC_SCAN_AN2|ADC_SCAN_AN3/*|ADC_SCAN_AN4|ADC_SCAN_AN5|ADC_SCAN_AN6|ADC_SCAN_AN7*/
	);
	SetPriorityIntADC1(INT_PRI_4);
	EnableIntADC1;
#endif

    return TRUE;
} // InitializeSystem

void __attribute__ ((interrupt,no_auto_psv)) _ADC1Interrupt(void)
{
	ADC1_Clear_Intr_Status_Bit;
	advalue[0]=ADC1BUF0;
	advalue[1]=ADC1BUF1;
}

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_BLUETOOTH_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch (event)
    {
        case EVENT_BLUETOOTH_ATTACH:
            if (size == sizeof(BLUETOOTH_DEVICE_ID))
            {
                deviceAddress   = ((BLUETOOTH_DEVICE_ID *)data)->deviceAddress;
                log_info( "Bluetooth device attached - event, deviceAddress=%d\r\n",deviceAddress );
                #ifdef USB_BLUETOOTH_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((BLUETOOTH_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        xputc( ((BLUETOOTH_DEVICE_ID *)data)->serialNumber[i] );
                    }
	                xputs( "\r\n" );
                #endif
                return TRUE;
            }
            break;

        case EVENT_BLUETOOTH_DETACH:
            deviceAddress   = 0;
            log_info( "Bluetooth device detached - event\r\n" );
            return TRUE;

        case EVENT_BLUETOOTH_TX_CMD_DONE:           // The main state machine will poll the driver.
            return TRUE;

        case EVENT_BLUETOOTH_TX_ACL_DONE:           // The main state machine will poll the driver.
            return TRUE;

        case EVENT_BLUETOOTH_RX_EVENT_DONE:
			if(*(DWORD*)data){
				event_bluetooth_rxEvent_done((WORD)(*(DWORD*)data));
			}
            return TRUE;

        case EVENT_BLUETOOTH_RX_ACL_DONE:
			if(*(DWORD*)data){
				event_bluetooth_rxAcl_done((WORD)(*(DWORD*)data));
			}
            return TRUE;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;

        case EVENT_HUB_ATTACH:
            log_info( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            log_info( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            log_info( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            log_info( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            log_info( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            log_info( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;

} // USB_ApplicationEventHandler

int main ( void )
{
	RestartWatchdog();

    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        log_info( "\r\n\r\nCould not initialize " LOCAL_NAME " - system.  Halting.\r\n\r\n" );
        while (1);
    }
	xdev_out(UART2PutChar);
    if ( USBHostInit(0) == TRUE )
    {
        log_info( "\r\n\r\n***** " LOCAL_NAME " Initialized *****\r\n\r\n" );
    }
    else
    {
        log_info( "\r\n\r\nCould not initialize " LOCAL_NAME " - USB.  Halting.\r\n\r\n" );
        while (1);
    }

    // Main Processing Loop
    while(!deviceAddress)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();

        //if (CheckForNewAttach()){
		//	break;
        //}
    }

	DelayMs(1000);

	blemain();

    return 0;

} // main


void hal_tick_init(void)
{
	usb_tick_handler=NULL;
}

void hal_tick_set_handler(void (*tick_handler)(void))
{
	usb_tick_handler=tick_handler;
}

void hal_tick_event(void)
{
	timer_counter++;

	RestartWatchdog();

	if(!startup_state && timer_counter>5000){
		Reset();
	}

	if(connection_status){
        if(timer_counter&0x100){
            led2_off();
        }else{
            led2_on();
        }
	}else{
		led2_on();
	}

#if	0
	if(led1_on_state && (timer_counter-led1_on_count)>LED1_ON_COUNT){
		led1_off();
		led1_on_state=0;
	}
#endif

	// call tick handler
	if(usb_tick_handler){
		usb_tick_handler();
	}
}

/*************************************************************************
 * EOF main.c
 */

